#lang racket

(require "bv.rkt" "verifier.rkt")
(provide (all-defined-out))

; Verify all benchmarks:
(define (verify-all)
 (run-verifier max1 max2)
 (run-verifier max1 max3)
 (run-verifier floor-of-ave1 floor-of-ave2)
 (run-verifier flip-right-01 flip-right-02)
 (run-verifier isolate-right-01 isolate-right-02))

(define (run-verifier f1 f2)
  (printf "Verifying: ~a ≡ ~a\n" (object-name f1) (object-name f2))
  (let-values ([(outs cpu real gc) (time-apply verify (list f1 f2))])
    (define out (car outs))
    (if (equal? out 'EQUIVALENT)
        (printf "  Outcome: EQUIVALENT\n")
        (printf "  Outcome: counterexample = ~a\n" out))
    (printf "  Time (ms): ~a\n" real)))

(define-fragment (max1 x y)
  (return (if (bvsgt x y) x y)))

(define-fragment (max2 x y)
  (define o1 (bvxor x y))
  (define o2 (if (bvuge x y) 1 0))
  (define o3 (bvneg o2))
  (define o4 (bvand o1 o2))
  (return (bvxor o3 y)))

(define-fragment (max3 x y)
  (define o1 (bvxor x y))
  (define o2 (if (bvsge y x) 1 0))
  (define o3 (bvneg o2))
  (define o4 (bvand o1 o3))
  (return (bvxor o4 x)))

(define-fragment (floor-of-ave1 x y)
  (define a (bvlshr x 1))
  (define b (bvlshr y 1))
  (define c (bvadd a b))
  (set! a (bvand 1 x))
  (set! b (bvand 1 y))
  (return (bvadd c (bvand a b))))

(define-fragment (floor-of-ave2 x y)
  (define o1 (bvand x y))
  (define o2 (bvxor x y))
  (define o3 (bvlshr o2 1))
  (return (bvadd o1 o3)))

(define-fragment (flip-right-01 x)
  (define o0 (bvand 1 x))
  (define o1 (= 0 o0))
  (define o2 (bvor 1 x))
  (define o3 (bvand 2 x))
  (define o4 (= 0 o3))
  (define o5 (bvor 2 x))
  (define o6 (bvand 4 x))
  (define o7 (= 0 o6))
  (define o8 (bvor 4 x))
  (define o9 (bvand 8 x))
  (define o10 (= 0 o9))
  (define o11 (bvor 8 x))
  (define o12 (bvand 16 x))
  (define o13 (= 0 o12))
  (define o14 (bvor 16 x))
  (define o15 (bvand 32 x))
  (define o16 (= 0 o15))
  (define o17 (bvor 32 x))
  (define o18 (bvand 64 x))
  (define o19 (= 0 o18))
  (define o20 (bvor 64 x))
  (define o21 (bvand 128 x))
  (define o22 (= 0 o21))
  (define o23 (bvor 128 x))
  (define o24 (bvand 256 x))
  (define o25 (= 0 o24))
  (define o26 (bvor 256 x))
  (define o27 (bvand 512 x))
  (define o28 (= 0 o27))
  (define o29 (bvor 512 x))
  (define o30 (bvand 1024 x))
  (define o31 (= 0 o30))
  (define o32 (bvor 1024 x))
  (define o33 (bvand 2048 x))
  (define o34 (= 0 o33))
  (define o35 (bvor 2048 x))
  (define o36 (bvand 4096 x))
  (define o37 (= 0 o36))
  (define o38 (bvor 4096 x))
  (define o39 (bvand 8192 x))
  (define o40 (= 0 o39))
  (define o41 (bvor 8192 x))
  (define o42 (bvand 16384 x))
  (define o43 (= 0 o42))
  (define o44 (bvor 16384 x))
  (define o45 (bvand 32768 x))
  (define o46 (= 0 o45))
  (define o47 (bvor 32768 x))
  (define o48 (bvand 65536 x))
  (define o49 (= 0 o48))
  (define o50 (bvor 65536 x))
  (define o51 (bvand 131072 x))
  (define o52 (= 0 o51))
  (define o53 (bvor 131072 x))
  (define o54 (bvand 262144 x))
  (define o55 (= 0 o54))
  (define o56 (bvor 262144 x))
  (define o57 (bvand 524288 x))
  (define o58 (= 0 o57))
  (define o59 (bvor 524288 x))
  (define o60 (bvand 1048576 x))
  (define o61 (= 0 o60))
  (define o62 (bvor 1048576 x))
  (define o63 (bvand 2097152 x))
  (define o64 (= 0 o63))
  (define o65 (bvor 2097152 x))
  (define o66 (bvand 4194304 x))
  (define o67 (= 0 o66))
  (define o68 (bvor 4194304 x))
  (define o69 (bvand 8388608 x))
  (define o70 (= 0 o69))
  (define o71 (bvor 8388608 x))
  (define o72 (bvand 16777216 x))
  (define o73 (= 0 o72))
  (define o74 (bvor 16777216 x))
  (define o75 (bvand 33554432 x))
  (define o76 (= 0 o75))
  (define o77 (bvor 33554432 x))
  (define o78 (bvand 67108864 x))
  (define o79 (= 0 o78))
  (define o80 (bvor 67108864 x))
  (define o81 (bvand 134217728 x))
  (define o82 (= 0 o81))
  (define o83 (bvor 134217728 x))
  (define o84 (bvand 268435456 x))
  (define o85 (= 0 o84))
  (define o86 (bvor 268435456 x))
  (define o87 (bvand 536870912 x))
  (define o88 (= 0 o87))
  (define o89 (bvor 536870912 x))
  (define o90 (bvand 1073741824 x))
  (define o91 (= 0 o90))
  (define o92 (bvor 1073741824 x))
  (define o93 (bvand -2147483648 x))
  (define o94 (= 0 o93))
  (define o95 (bvor -2147483648 x))
  (define o96 (if o94 o95 -1))
  (define o97 (if o91 o92 o96))
  (define o98 (if o88 o89 o97))
  (define o99 (if o85 o86 o98))
  (define o100 (if o82 o83 o99))
  (define o101 (if o79 o80 o100))
  (define o102 (if o76 o77 o101))
  (define o103 (if o73 o74 o102))
  (define o104 (if o70 o71 o103))
  (define o105 (if o67 o68 o104))
  (define o106 (if o64 o65 o105))
  (define o107 (if o61 o62 o106))
  (define o108 (if o58 o59 o107))
  (define o109 (if o55 o56 o108))
  (define o110 (if o52 o53 o109))
  (define o111 (if o49 o50 o110))
  (define o112 (if o46 o47 o111))
  (define o113 (if o43 o44 o112))
  (define o114 (if o40 o41 o113))
  (define o115 (if o37 o38 o114))
  (define o116 (if o34 o35 o115))
  (define o117 (if o31 o32 o116))
  (define o118 (if o28 o29 o117))
  (define o119 (if o25 o26 o118))
  (define o120 (if o22 o23 o119))
  (define o121 (if o19 o20 o120))
  (define o122 (if o16 o17 o121))
  (define o123 (if o13 o14 o122))
  (define o124 (if o10 o11 o123))
  (define o125 (if o7 o8 o124))
  (define o126 (if o4 o5 o125))
  (return (if o1 o2 o126)))

(define-fragment (flip-right-02 y)
  (define o1 (bvadd y 1))
  (return (bvor y o1)))

(define-fragment (isolate-right-01 z)
  (define o0 (bvand 1 z))
  (define o1 (= 0 o0))
  (define o2 (bvor 1 z))
  (define o3 (bvand 2 z))
  (define o4 (= 0 o3))
  (define o5 (bvor 2 z))
  (define o6 (bvand 4 z))
  (define o7 (= 0 o6))
  (define o8 (bvor 4 z))
  (define o9 (bvand 8 z))
  (define o10 (= 0 o9))
  (define o11 (bvor 8 z))
  (define o12 (bvand 16 z))
  (define o13 (= 0 o12))
  (define o14 (bvor 16 z))
  (define o15 (bvand 32 z))
  (define o16 (= 0 o15))
  (define o17 (bvor 32 z))
  (define o18 (bvand 64 z))
  (define o19 (= 0 o18))
  (define o20 (bvor 64 z))
  (define o21 (bvand 128 z))
  (define o22 (= 0 o21))
  (define o23 (bvor 128 z))
  (define o24 (bvand 256 z))
  (define o25 (= 0 o24))
  (define o26 (bvor 256 z))
  (define o27 (bvand 512 z))
  (define o28 (= 0 o27))
  (define o29 (bvor 512 z))
  (define o30 (bvand 1024 z))
  (define o31 (= 0 o30))
  (define o32 (bvor 1024 z))
  (define o33 (bvand 2048 z))
  (define o34 (= 0 o33))
  (define o35 (bvor 2048 z))
  (define o36 (bvand 4096 z))
  (define o37 (= 0 o36))
  (define o38 (bvor 4096 z))
  (define o39 (bvand 8192 z))
  (define o40 (= 0 o39))
  (define o41 (bvor 8192 z))
  (define o42 (bvand 16384 z))
  (define o43 (= 0 o42))
  (define o44 (bvor 16384 z))
  (define o45 (bvand 32768 z))
  (define o46 (= 0 o45))
  (define o47 (bvor 32768 z))
  (define o48 (bvand 65536 z))
  (define o49 (= 0 o48))
  (define o50 (bvor 65536 z))
  (define o51 (bvand 131072 z))
  (define o52 (= 0 o51))
  (define o53 (bvor 131072 z))
  (define o54 (bvand 262144 z))
  (define o55 (= 0 o54))
  (define o56 (bvor 262144 z))
  (define o57 (bvand 524288 z))
  (define o58 (= 0 o57))
  (define o59 (bvor 524288 z))
  (define o60 (bvand 1048576 z))
  (define o61 (= 0 o60))
  (define o62 (bvor 1048576 z))
  (define o63 (bvand 2097152 z))
  (define o64 (= 0 o63))
  (define o65 (bvor 2097152 z))
  (define o66 (bvand 4194304 z))
  (define o67 (= 0 o66))
  (define o68 (bvor 4194304 z))
  (define o69 (bvand 8388608 z))
  (define o70 (= 0 o69))
  (define o71 (bvor 8388608 z))
  (define o72 (bvand 16777216 z))
  (define o73 (= 0 o72))
  (define o74 (bvor 16777216 z))
  (define o75 (bvand 33554432 z))
  (define o76 (= 0 o75))
  (define o77 (bvor 33554432 z))
  (define o78 (bvand 67108864 z))
  (define o79 (= 0 o78))
  (define o80 (bvor 67108864 z))
  (define o81 (bvand 134217728 z))
  (define o82 (= 0 o81))
  (define o83 (bvor 134217728 z))
  (define o84 (bvand 268435456 z))
  (define o85 (= 0 o84))
  (define o86 (bvor 268435456 z))
  (define o87 (bvand 536870912 z))
  (define o88 (= 0 o87))
  (define o89 (bvor 536870912 z))
  (define o90 (bvand 1073741824 z))
  (define o91 (= 0 o90))
  (define o92 (bvor 1073741824 z))
  (define o93 (bvand -2147483648 z))
  (define o94 (= 0 o93))
  (define o95 (bvor -2147483648 z))
  (define o96 (if o94 o95 -1))
  (define o97 (if o91 o92 o96))
  (define o98 (if o88 o89 o97))
  (define o99 (if o85 o86 o98))
  (define o100 (if o82 o83 o99))
  (define o101 (if o79 o80 o100))
  (define o102 (if o76 o77 o101))
  (define o103 (if o73 o74 o102))
  (define o104 (if o70 o71 o103))
  (define o105 (if o67 o68 o104))
  (define o106 (if o64 o65 o105))
  (define o107 (if o61 o62 o106))
  (define o108 (if o58 o59 o107))
  (define o109 (if o55 o56 o108))
  (define o110 (if o52 o53 o109))
  (define o111 (if o49 o50 o110))
  (define o112 (if o46 o47 o111))
  (define o113 (if o43 o44 o112))
  (define o114 (if o40 o41 o113))
  (define o115 (if o37 o38 o114))
  (define o116 (if o34 o35 o115))
  (define o117 (if o31 o32 o116))
  (define o118 (if o28 o29 o117))
  (define o119 (if o25 o26 o118))
  (define o120 (if o22 o23 o119))
  (define o121 (if o19 o20 o120))
  (define o122 (if o16 o17 o121))
  (define o123 (if o13 o14 o122))
  (define o124 (if o10 o11 o123))
  (define o125 (if o7 o8 o124))
  (define o126 (if o4 o5 o125))
  (return (if o1 o2 o126)))

(define-fragment (isolate-right-02 u)
  (define o0 (bvnot u)) 
  (define o1 (bvadd 1 u)) 
  (define o2 (bvand o0 o1))
  (define o3 (bvshl 1 31))
  (return (if (= u o3) 1 o2)))

(verify-all)